@@ -1,10 +1,10 @@
 /*
  * Program name:  equake
- * Version:	  1.03.3
+ * Version:	  1.03.5
  * Source file:	  equake_applet.c  
  * Description:	  Equake pulls data about earthquakes and displays it in interesting ways
  *
- * Copyright (C) 2012-2013 Jeroen van Aart
+ * Copyright (C) 2012-2014 Jeroen van Aart
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -79,7 +79,6 @@ void equake_getprefs(struct Equake_Data *equakedata)
   FILE *fp;
   char *buf;
   size_t nbytes=100;
-  ssize_t bytesread;
   int val;
 
   fp=fopen(equakedata->configfile, "r");
@@ -88,7 +87,7 @@ void equake_getprefs(struct Equake_Data *equakedata)
 
   buf=(char*)malloc(nbytes+1);
   if (buf==NULL)
-    exit(-1);
+    return;
 
   if (getline(&buf, &nbytes, fp)!=-1)
   {
@@ -306,7 +305,7 @@ void equake_saveprefs(struct Equake_Data *equakedata, FILE *fp)
 static int button_press_callback(GtkWidget *event_box, GdkEventButton *event, gpointer data)
 {
   char url[512];
-  GError *error;
+  
   /* only do something if left mouse button was released */
   if ((event->type==GDK_BUTTON_PRESS) && (event->button==1))
   {
@@ -379,11 +378,17 @@ static void equake_size_changed(XfcePanelPlugin *applet, int size, struct Equake
  */
 static void equake_applet_fill(XfcePanelPlugin *applet)
 {
-  GtkWidget *label, *logo, *button, *event_box;
-  GtkWidget *menu_update, *menu_details, *menu_map, *menu_didyoufeelit, *menu_lasthour, *menu_lastday;
+  GtkWidget *label, *logo, *event_box;
+  GtkWidget *menu_update, *menu_details, *menu_map, *menu_didyoufeelit, *menu_lasthour, *menu_lastday, *menu_lastweek;
   GdkPixbuf *quake_logo;
  
   struct Equake_Data *equakedata=g_malloc0(sizeof(struct Equake_Data));
+  /* 
+   * we exit when this alloc returns NULL, since without this continuing the program is pointless
+   * in other cases the program doesn't depend on the alloc succeeding right then and there
+   * in all other cases the function returns when alloc returns NULL
+   * although an exit() is may be preferred I suspect a real out of memory is so rare these days that a quick exit() is excessive
+   */
   if(equakedata==NULL)
     exit(-1);
  
@@ -459,6 +464,12 @@ static void equake_applet_fill(XfcePanelPlugin *applet)
   g_signal_connect(G_OBJECT(menu_lastday), "activate", G_CALLBACK(display_lastday), equakedata);
   xfce_panel_plugin_menu_insert_item(applet, GTK_MENU_ITEM(menu_lastday));
 
+  menu_lastweek=gtk_image_menu_item_new_from_stock("gtk-index", NULL);
+  gtk_menu_item_set_label((GtkMenuItem *)menu_lastweek, "Last _week (slow)");
+  gtk_widget_show(menu_lastweek);
+  g_signal_connect(G_OBJECT(menu_lastweek), "activate", G_CALLBACK(display_lastweek), equakedata);
+  xfce_panel_plugin_menu_insert_item(applet, GTK_MENU_ITEM(menu_lastweek));
+
   gtk_widget_show_all(GTK_WIDGET(applet));
 
   startequake(applet, label, logo, equakedata);
@@ -473,11 +484,11 @@ XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL(equake_applet_fill);
  *********************************************
  *
  * Program name:  equake
- * Version:	  1.03
+ * Version:	  1.03.4
  * Source file:	  equake_menu.c  
  * Description:	  Equake pulls data about earthquakes and displays it in interesting ways
  *
- * Copyright (C) 2012-2013 Jeroen van Aart
+ * Copyright (C) 2012-2014 Jeroen van Aart
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -509,7 +520,6 @@ static void do_update(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 static void load_details(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 {
   char url[URLSIZE];
-  GError *error;
 
   if (equakedata->posperiod==HOURLY)
   {
@@ -543,7 +553,6 @@ static void load_details(XfcePanelPlugin *applet, struct Equake_Data *equakedata
 static void load_map(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 {
   char url[URLSIZE];
-  GError *error;
 
   if (equakedata->posperiod==HOURLY)
   {
@@ -553,9 +562,7 @@ static void load_map(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
       display_url(url);
     }
     else
-    {
       display_dialog("No data available at this time");
-    }
   }
   else
   {
@@ -565,16 +572,13 @@ static void load_map(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
       display_url(url);
     }
     else
-    {
       display_dialog("No data available at this time");
-    }
   }
 }
 
 static void load_didyoufeelit(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 {
   char url[URLSIZE];
-  GError *error;
 
   if (equakedata->posperiod==HOURLY)
   {
@@ -604,19 +608,17 @@ static void display_properties_dialog(XfcePanelPlugin *applet, struct Equake_Dat
   int polltime, willalert, alertheavy, sigfilemag, monitormag;
   FILE *fp;
 
-  GtkWidget *label, *content_area, *polltime_select, *willalert_select, *alertheavy_select, *sigfilemag_select, *monitormag_select, *image_about, *image_frame_about;
+  GtkWidget *polltime_select, *willalert_select, *alertheavy_select, *sigfilemag_select, *monitormag_select, *image_about;
   GtkWidget *labelpoll, *labelwillalert, *labelalertheavy, *labelsigfile, *labelmonitormag, *sep, *sep2;
 
  /* block menu from being used whilst properties dialog is open */
   xfce_panel_plugin_block_menu(applet);
 
   equakedata->prefsdialog=gtk_dialog_new_with_buttons(PREFSWINDOW, NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
-  content_area=gtk_dialog_get_content_area(GTK_DIALOG (equakedata->prefsdialog));
 
   sep=gtk_hseparator_new();
   sep2=gtk_hseparator_new();
 
-  label=gtk_label_new(PREFSWINDOW);
   labelpoll=gtk_label_new(PREFPOLLTEXT);
   labelwillalert=gtk_label_new(PREFALERTEXT);
   labelalertheavy=gtk_label_new(PREFALERTHEAVYTEXT);
@@ -644,13 +646,11 @@ static void display_properties_dialog(XfcePanelPlugin *applet, struct Equake_Dat
   /* properties_index[] holds default or previous combo box indexes */
   gtk_combo_box_set_active((GtkComboBox*)polltime_select, equakedata->properties_index[0]);
 
-
   gtk_combo_box_insert_text((GtkComboBox*)willalert_select, 0, "No");
   gtk_combo_box_insert_text((GtkComboBox*)willalert_select, 1, "Yes");
 
   gtk_combo_box_set_active((GtkComboBox*)willalert_select, equakedata->properties_index[1]);
 
-
   gtk_combo_box_insert_text((GtkComboBox*)alertheavy_select, 0, "3");
   gtk_combo_box_insert_text((GtkComboBox*)alertheavy_select, 1, "4");
   gtk_combo_box_insert_text((GtkComboBox*)alertheavy_select, 2, "5");
@@ -660,7 +660,6 @@ static void display_properties_dialog(XfcePanelPlugin *applet, struct Equake_Dat
 
   gtk_combo_box_set_active((GtkComboBox*)alertheavy_select, equakedata->properties_index[2]);
 
-
   gtk_combo_box_insert_text((GtkComboBox*)sigfilemag_select, 0, "0");
   gtk_combo_box_insert_text((GtkComboBox*)sigfilemag_select, 1, "2");
   gtk_combo_box_insert_text((GtkComboBox*)sigfilemag_select, 2, "3");
@@ -672,7 +671,6 @@ static void display_properties_dialog(XfcePanelPlugin *applet, struct Equake_Dat
 
   gtk_combo_box_set_active((GtkComboBox*)sigfilemag_select, equakedata->properties_index[3]);
 
-
   gtk_combo_box_insert_text((GtkComboBox*)monitormag_select, 0, "1");
   gtk_combo_box_insert_text((GtkComboBox*)monitormag_select, 1, "2");
   gtk_combo_box_insert_text((GtkComboBox*)monitormag_select, 2, "3");
@@ -909,31 +907,49 @@ static void display_about_dialog(XfcePanelPlugin *applet, struct Equake_Data *eq
 }
 
 /* 
- * displays last day, will first load data for last day
- * then just runs display_lasthour to display it in a dialog
+ * displays last hour/day/week, will first load data for last hour/day/week
+ * then just runs display_lasthour/day/week to display it in a dialog
  */
+static void display_lastweek(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
+{
+  if (getquakedata(equakedata, WEEKLY)==1)
+  {
+    if (processdata(equakedata, WEEKLY)==-1)
+    {
+      perror("Failed to process data"); /* non fatal error, hopefully */
+      display_dialog("No data available at this time");
+      return;
+    }
+    display_lastperiod(equakedata, WEEKLY);
+  }
+}
+
 static void display_lastday(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 {
   if (getquakedata(equakedata, DAILY)==1)
+  {
     if (processdata(equakedata, DAILY)==-1)
     {
       perror("Failed to process data"); /* non fatal error, hopefully */
       display_dialog("No data available at this time");
       return;
     }
-  display_lastperiod(equakedata, DAILY);
+    display_lastperiod(equakedata, DAILY);
+  }
 }
-
+  
 static void display_lasthour(XfcePanelPlugin *applet, struct Equake_Data *equakedata)
 {
   if (getquakedata(equakedata, HOURLY)==1)
+  {
     if (processdata(equakedata, HOURLY)==-1)
     {
       perror("Failed to process data"); /* non fatal error, hopefully */
       display_dialog("No data available at this time");
       return;
     }
-  display_lastperiod(equakedata, HOURLY);
+    display_lastperiod(equakedata, HOURLY);
+  }
 }
 
 /*
@@ -952,6 +968,8 @@ static void fill_text_period(struct Equake_Data *equakedata, char *running, char
     {
       /* create url to be displayed */
       snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformatday[n].eqid, QUAKEPATH_AF);
+      /* create map url */
+      snprintf(mapurl, URLSIZE, "%s%s%s%s%s%s%s%s%s%s%s%s", "http://", MAPURL1, equakedata->equakeformatday[n].lat, MAPURL2, equakedata->equakeformatday[n].lon, MAPURL3, "Earthquake+Magnitude:+", equakedata->equakeformatday[n].magnitude, "+Region:+", equakedata->equakeformatday[n].region, "+Rendered+by+equake+applet", MAPURL4);
 
       /* convert magnitude to double */
       if(equakedata->equakeformatday[n].magnitude!=NULL)
@@ -963,51 +981,90 @@ static void fill_text_period(struct Equake_Data *equakedata, char *running, char
       {
         /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
         if (mag>=DAILYHEAVY)
-          snprintf(running, strlen(equakedata->bufferdaily), "<b>Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a></b>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url, url, url);
+          snprintf(running, strlen(equakedata->bufferdaily), "<b>Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a></b>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].datetime_local, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url, url, url, mapurl, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].region);
         else
-          snprintf(running, strlen(equakedata->bufferdaily), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url, url, url);
+          snprintf(running, strlen(equakedata->bufferdaily), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].datetime_local, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url, url, url, mapurl, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].region);
       }
       else
       {
         /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
         if (mag>=DAILYHEAVY)
-          snprintf(running, strlen(equakedata->bufferdaily), "<b>Earthquake Magnitude: %s</b>\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a></b>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url, url, url);
+          snprintf(running, strlen(equakedata->bufferdaily), "<b>Earthquake Magnitude: %s</b>\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s</b>\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].datetime_local, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url);
         else
-          snprintf(running, strlen(equakedata->bufferdaily), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url);
+          snprintf(running, strlen(equakedata->bufferdaily), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformatday[n].magnitude, equakedata->equakeformatday[n].datetime, equakedata->equakeformatday[n].datetime_local, equakedata->equakeformatday[n].region, equakedata->equakeformatday[n].lat, equakedata->equakeformatday[n].lon, equakedata->equakeformatday[n].depth, url);
       }
       strncat(periodtext, running, strlen(running));
     }
   }
   else
   {
-    for (n=0;n<=equakedata->quake_history_count;n++)
+    if (period==WEEKLY)
     {
-      /* create url to be displayed */
-      snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformat[n].eqid, QUAKEPATH_AF);
-
-     /* convert magnitude to double */
-      if(equakedata->equakeformat[n].magnitude!=NULL)
-        mag=atof(equakedata->equakeformat[n].magnitude);
-      else
-        mag=0;
-
-      if ((gtk_major_version>=2) && (gtk_minor_version>=18))
+      for (n=0;n<=equakedata->quake_history_countweekly;n++)
       {
-        /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
-        if (mag>=DAILYHEAVY)
-          snprintf(running, strlen(equakedata->buffer), "<b>Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a></b>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url, url, url);
+        /* create url to be displayed */
+        snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformatweek[n].eqid, QUAKEPATH_AF);
+        /* create map url */
+        snprintf(mapurl, URLSIZE, "%s%s%s%s%s%s%s%s%s%s%s%s", "http://", MAPURL1, equakedata->equakeformatweek[n].lat, MAPURL2, equakedata->equakeformatweek[n].lon, MAPURL3, "Earthquake+Magnitude:+", equakedata->equakeformatweek[n].magnitude, "+Region:+", equakedata->equakeformatweek[n].region, "+Rendered+by+equake+applet", MAPURL4);
+
+        /* convert magnitude to double */
+        if(equakedata->equakeformatweek[n].magnitude!=NULL)
+          mag=atof(equakedata->equakeformatweek[n].magnitude);
         else
-          snprintf(running, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url, url, url);
+          mag=0;
+
+        if ((gtk_major_version>=2) && (gtk_minor_version>=18))
+        {
+          /* if magnitude is bigger or equal to WEEKLYHEAVY make magnitude bold */
+          if (mag>=DAILYHEAVY)
+            snprintf(running, strlen(equakedata->bufferweekly), "<b>Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a></b>\n\n", equakedata->equakeformatweek[n].magnitude, equakedata->equakeformatweek[n].datetime, equakedata->equakeformatweek[n].datetime_local, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].lat, equakedata->equakeformatweek[n].lon, equakedata->equakeformatweek[n].depth, url, url, url, mapurl, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].region);
+          else
+            snprintf(running, strlen(equakedata->bufferweekly), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a>\n\n", equakedata->equakeformatweek[n].magnitude, equakedata->equakeformatweek[n].datetime, equakedata->equakeformatweek[n].datetime_local, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].lat, equakedata->equakeformatweek[n].lon, equakedata->equakeformatweek[n].depth, url, url, url, mapurl, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].region);
+        }
+        else
+        {
+          /* if magnitude is bigger or equal to WEEKLYHEAVY make magnitude bold */
+          if (mag>=DAILYHEAVY)
+            snprintf(running, strlen(equakedata->bufferweekly), "<b>Earthquake Magnitude: %s</b>\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s</b>\n\n", equakedata->equakeformatweek[n].magnitude, equakedata->equakeformatweek[n].datetime, equakedata->equakeformatweek[n].datetime_local, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].lat, equakedata->equakeformatweek[n].lon, equakedata->equakeformatweek[n].depth, url);
+          else
+            snprintf(running, strlen(equakedata->bufferweekly), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformatweek[n].magnitude, equakedata->equakeformatweek[n].datetime, equakedata->equakeformatweek[n].datetime_local, equakedata->equakeformatweek[n].region, equakedata->equakeformatweek[n].lat, equakedata->equakeformatweek[n].lon, equakedata->equakeformatweek[n].depth, url);
+        }
+        strncat(periodtext, running, strlen(running));
       }
-      else
+    }
+    else
+    {
+      for (n=0;n<=equakedata->quake_history_count;n++)
       {
-        /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
-        if (mag>=DAILYHEAVY)
-          snprintf(running, strlen(equakedata->buffer), "<b>Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s</b>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url);
+        /* create url to be displayed */
+        snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformat[n].eqid, QUAKEPATH_AF);
+	/* create map url */
+        snprintf(mapurl, URLSIZE, "%s%s%s%s%s%s%s%s%s%s%s%s", "http://", MAPURL1, equakedata->equakeformat[n].lat, MAPURL2, equakedata->equakeformat[n].lon, MAPURL3, "Earthquake+Magnitude:+", equakedata->equakeformat[n].magnitude, "+Region:+", equakedata->equakeformat[n].region, "+Rendered+by+equake+applet", MAPURL4);
+
+	/* convert magnitude to double */
+        if(equakedata->equakeformat[n].magnitude!=NULL)
+          mag=atof(equakedata->equakeformat[n].magnitude);
         else
-          snprintf(running, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url);
+          mag=0;
+
+	if ((gtk_major_version>=2) && (gtk_minor_version>=18))
+        {
+          /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
+          if (mag>=DAILYHEAVY)
+	    snprintf(running, strlen(equakedata->buffer), "<b>Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a></b>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].datetime_local, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url, url, url, mapurl, equakedata->equakeformat[n].region, equakedata->equakeformat[n].region);
+	  else
+	    snprintf(running, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: <a href=\"%s\" title=\"%s\">%s</a>\n<a href=\"%s\" title=\"Detailed Map %s\">Detailed Map %s</a>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].datetime_local, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url, url, url, mapurl, equakedata->equakeformat[n].region, equakedata->equakeformat[n].region);
+        }
+        else
+        {
+	  /* if magnitude is bigger or equal to DAILYHEAVY make magnitude bold */
+	  if (mag>=DAILYHEAVY)
+	    snprintf(running, strlen(equakedata->buffer), "<b>Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s</b>\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].datetime_local, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url);
+          else
+            snprintf(running, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformat[n].magnitude, equakedata->equakeformat[n].datetime, equakedata->equakeformat[n].datetime_local, equakedata->equakeformat[n].region, equakedata->equakeformat[n].lat, equakedata->equakeformat[n].lon, equakedata->equakeformat[n].depth, url);
+        }
+	strncat(periodtext, running, strlen(running));
       }
-      strncat(periodtext, running, strlen(running));
     }
   }
 }
@@ -1015,9 +1072,8 @@ static void fill_text_period(struct Equake_Data *equakedata, char *running, char
 static void display_period_window(char *text, int period)
 {
   int screenheight;
-  GtkWidget *dialog, *scrolled_window, *window, *label, *button;
+  GtkWidget *scrolled_window, *window, *label, *button;
   GdkScreen *screen;
-  PangoContext *label_context;
 
   /* get screen height */
   screen=gdk_screen_get_default();
@@ -1025,6 +1081,8 @@ static void display_period_window(char *text, int period)
 
   /* allocate memory for the GtkRequisition funtion below */
   GtkRequisition *requisition=alloca(sizeof(GtkRequisition));
+  if (requisition==NULL)
+    return;
   
   /*
    * we're creating a window in which we place a scrolled window with a label
@@ -1034,9 +1092,11 @@ static void display_period_window(char *text, int period)
 
   /* create the window */
   window=gtk_dialog_new();
-  //g_signal_connect(window, "destroy", G_CALLBACK(destroy), NULL);
+  
   if (period==DAILY)
     gtk_window_set_title(GTK_WINDOW (window), RECDAYTXT);
+  else if (period==WEEKLY)
+    gtk_window_set_title(GTK_WINDOW (window), RECWEEKTXT);
   else
     gtk_window_set_title(GTK_WINDOW (window), RECHOURTXT);
 
@@ -1053,7 +1113,7 @@ static void display_period_window(char *text, int period)
 
   /* label */
   label=gtk_label_new(text);
-  gtk_label_set_markup((GtkLabel *)label, text);
+  gtk_label_set_use_markup((GtkLabel *)label, TRUE);
   
   gtk_label_set_selectable((GtkLabel *)label, TRUE);
   gtk_label_set_line_wrap((GtkLabel *)label, FALSE);
@@ -1074,16 +1134,13 @@ static void display_period_window(char *text, int period)
 
   gtk_widget_size_request(label, requisition);
 
-  if (period==DAILY)
-    gtk_widget_set_size_request(window, requisition->width+60, screenheight-100);
-  else
-  {
   /* make window not larger than screen height and keep 50 pixels headroom in case of larger/multiple panels */
-  if ((requisition->height+100)<(screenheight-100))
+  if (period!=HOURLY)
+    gtk_widget_set_size_request(window, requisition->width+60, screenheight-100);
+  else if ((requisition->height+100)<(screenheight-100))
     gtk_widget_set_size_request(window, requisition->width+60, requisition->height+100);
   else
     gtk_widget_set_size_request(window, requisition->width+60, screenheight-100);
-  }
 
   gtk_widget_show(window);
 }
@@ -1091,27 +1148,32 @@ static void display_period_window(char *text, int period)
 
 static void display_lastperiod(struct Equake_Data *equakedata, int period)
 {
-  int n;
-  char url[URLSIZE];
-  double mag;
-
   /* allocate memory depending on the period, hourly is default */
   char *lastday=alloca(sizeof(equakedata->bufferdaily)+1); /* there's not more memory needed than what's in the buffer and normally much less */
   if (lastday==NULL)
-    exit(-1);
+    return;
   char *runningday=alloca(sizeof(equakedata->bufferdaily)+1);
   if (runningday==NULL)
-    exit(-1);
+    return;
+
+  char *lastweek=alloca(sizeof(equakedata->bufferweekly)+1);
+  if (lastweek==NULL)
+    return;
+  char *runningweek=alloca(sizeof(equakedata->bufferweekly)+1);
+  if (runningweek==NULL)
+    return;
 
   memset(lastday, 0, strlen(lastday));
   memset(runningday, 0, strlen(runningday));
+  memset(lastweek, 0, strlen(lastweek));
+  memset(runningweek, 0, strlen(runningweek));
     
   char *lasthour=alloca(sizeof(equakedata->buffer)+1); /* there's not more memory needed than what's in the buffer and normally much less */
   if (lasthour==NULL)
-    exit(-1);
+    return;
   char *running=alloca(sizeof(equakedata->buffer)+1);
   if (running==NULL)
-    exit(-1);
+    return;
 
   memset(lasthour, 0, strlen(lasthour));
   memset(running, 0, strlen(running));
@@ -1123,11 +1185,19 @@ static void display_lastperiod(struct Equake_Data *equakedata, int period)
   }
   else
   {
-    if ((period!=DAILY) && (equakedata->quake_history_count<0))
+    if ((period==WEEKLY) && (equakedata->quake_history_countweekly<0))
     {
       display_dialog("No data available at this time");
       return;
     }
+    else
+    {
+      if ((period!=WEEKLY) && (period!=DAILY) && (equakedata->quake_history_count<0))
+      {
+        display_dialog("No data available at this time");
+        return;
+      }
+    }
   }
 
   if (period==DAILY)
@@ -1137,10 +1207,17 @@ static void display_lastperiod(struct Equake_Data *equakedata, int period)
   }
   else
   {
-    fill_text_period(equakedata, running, lasthour, period);
-    display_period_window(lasthour, period);
+    if (period==WEEKLY)
+    {
+      fill_text_period(equakedata, runningweek, lastweek, period);
+      display_period_window(lastweek, period);
+    }
+    else
+    {
+      fill_text_period(equakedata, running, lasthour, period);
+      display_period_window(lasthour, period);
+    }
   }
-
 }
 
 static void display_dialog(char *t)
