@@ -1,10 +1,10 @@
 /*
  * Program name:  equake
- * Version:	  1.03.3
+ * Version:	  1.03.5
  * Source file:	  equake_processdata.c
  * Description:	  Equake pulls data about earthquakes and displays it in interesting ways
  *
- * Copyright (C) 2012-2013 Jeroen van Aart
+ * Copyright (C) 2012-2014 Jeroen van Aart
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,17 +49,17 @@
 int processdata_daily(struct Equake_Data *equakedata)
 {
   const char delimiters[] = ",\"";
-  const char delimiterquote[] = "\""; // some fields are between quotes, with commas
-  char url[URLSIZE], *token;
+  const char delimiterquote[] = "\""; /* some fields are between quotes, with commas */
+  const char delimiterdate[]="TZ"; /* new date format has strings T and Z, we use it to separate date and time */
+  char *token, *dt1, *dt2, buf[URLSIZE];
   int n;
 
   char *running=alloca(sizeof(equakedata->bufferdaily)+1);
   if (running==NULL)
-    exit(-1);
+    return -1;
  
   equakedata->quake_history_countdaily=-1;
  
-  /*sprintf(running, "%s", equakedata->bufferdaily);*/
   snprintf(running, strlen(equakedata->bufferdaily), "%s", equakedata->bufferdaily);
 
   running=strstr(running, equakedata->formatstring); // find line with field names
@@ -94,92 +94,106 @@ int processdata_daily(struct Equake_Data *equakedata)
     if (token==NULL)
       break;
 
-    equakedata->equakeformatday[n].datetime=strndup(token, strlen(token));
-    if (equakedata->equakeformatday[n].datetime!="")
+    /* lets make date more readable by filtering out T and Z strings and adding some clarification */
+    dt1=strsep(&token, delimiterdate);
+    if (dt1==NULL)
+      break;
+    dt2=strsep(&token, delimiterdate);
+    if (dt1==NULL)
+      break;
+    snprintf(buf, URLSIZE, "%s%s%s%s%s", dt1, "  ", dt2, " ", TIMEZONE);
+
+    equakedata->equakeformatday[n].datetime=strndup(buf, strlen(buf));
+
+    if (equakedata->equakeformatday[n].datetime!=NULL)
     {
+      /* now that we have datetime lets convert it to local time */
+      if (convert_localtime(&equakedata->equakeformatday[n])==-1)
+        equakedata->equakeformatday[n].datetime_local=strndup(buf, strlen(buf));
+
       token=strsep(&running, delimiters);
       if (token==NULL)
         break;
 
       equakedata->equakeformatday[n].lat=strndup(token, strlen(token));
-      if (equakedata->equakeformatday[n].lat!="")
+      if (equakedata->equakeformatday[n].lat!=NULL)
       {
         token=strsep(&running, delimiters);
         if (token==NULL)
           break;
 
         equakedata->equakeformatday[n].lon=strndup(token, strlen(token));
-        if (equakedata->equakeformatday[n].lon!="")
+        if (equakedata->equakeformatday[n].lon!=NULL)
         {
           token=strsep(&running, delimiters); // eliminate whitespace 
           if (token==NULL)
             break;
 
           equakedata->equakeformatday[n].depth=strndup(token, strlen(token));
-          if (equakedata->equakeformatday[n].depth!="")
+          if (equakedata->equakeformatday[n].depth!=NULL)
           {
             token=strsep(&running, delimiters); // eliminate whitespace
             if (token==NULL)
               break;
 
             equakedata->equakeformatday[n].magnitude=strndup(token, strlen(token));
-            if (equakedata->equakeformatday[n].magnitude!="")
+            if (equakedata->equakeformatday[n].magnitude!=NULL)
             {
               token=strsep(&running, delimiters);
               if (token==NULL)
                 break;
 
               equakedata->equakeformatday[n].magnitude_type=strndup(token, strlen(token));
-              if (equakedata->equakeformatday[n].magnitude_type!="")
+              if (equakedata->equakeformatday[n].magnitude_type!=NULL)
               {
                 token=strsep(&running, delimiters);
                 if (token==NULL)
                   break;
 
                 equakedata->equakeformatday[n].nst=strndup(token, strlen(token));
-                if (equakedata->equakeformatday[n].nst!="")
+                if (equakedata->equakeformatday[n].nst!=NULL)
                 {
                   token=strsep(&running, delimiters);
                   if (token==NULL)
                     break;
 
                   equakedata->equakeformatday[n].gap=strndup(token, strlen(token));
-                  if (equakedata->equakeformatday[n].gap!="")
+                  if (equakedata->equakeformatday[n].gap!=NULL)
                   {
                     token=strsep(&running, delimiters); // eliminate whitespace
                     if (token==NULL)
                       break;
 
                     equakedata->equakeformatday[n].dmin=strndup(token, strlen(token));
-                    if (equakedata->equakeformatday[n].dmin!="")
+                    if (equakedata->equakeformatday[n].dmin!=NULL)
                     {
                       token=strsep(&running, delimiters); // eliminate whitespace
                       if (token==NULL)
                         break;
 
                       equakedata->equakeformatday[n].rms=strndup(token, strlen(token));
-                      if (equakedata->equakeformatday[n].rms!="")
+                      if (equakedata->equakeformatday[n].rms!=NULL)
                       {
                         token=strsep(&running, delimiters); // eliminate whitespace
                         if (token==NULL)
                           break;
 
                         equakedata->equakeformatday[n].src=strndup(token, strlen(token));
-                        if (equakedata->equakeformatday[n].src!="")
+                        if (equakedata->equakeformatday[n].src!=NULL)
                         {
                           token=strsep(&running, delimiters); // eliminate whitespace
                           if (token==NULL)
                             break;
  
                           equakedata->equakeformatday[n].eqid=strndup(token, strlen(token));
-                          if (equakedata->equakeformatday[n].eqid!="")
+                          if (equakedata->equakeformatday[n].eqid!=NULL)
                           {
                             token=strsep(&running, delimiters); // eliminate whitespace
                             if (token==NULL)
                               break;
 
                             equakedata->equakeformatday[n].updated=strndup(token, strlen(token));
-                            if (equakedata->equakeformatday[n].updated!="")
+                            if (equakedata->equakeformatday[n].updated!=NULL)
                             {
                               token=strsep(&running, delimiters); // eliminate whitespace
                               if (token==NULL)
@@ -222,6 +236,197 @@ int processdata_daily(struct Equake_Data *equakedata)
 
   return equakedata->quake_history_countdaily;
 }
+
+int processdata_weekly(struct Equake_Data *equakedata)
+{
+  const char delimiters[] = ",\"";
+  const char delimiterquote[] = "\""; /* some fields are between quotes, with commas */
+  const char delimiterdate[]="TZ"; /* new date format has strings T and Z, we use it to separate date and time */
+  char *token, *dt1, *dt2, buf[URLSIZE];
+  int n;
+
+  char *running=alloca(sizeof(equakedata->bufferweekly)+1);
+  if (running==NULL)
+    return -1;
+
+  equakedata->quake_history_countweekly=-1;
+
+  snprintf(running, strlen(equakedata->bufferweekly), "%s", equakedata->bufferweekly);
+
+  running=strstr(running, equakedata->formatstring); // find line with field names
+  if (running==NULL)
+    return -1; /* non fatal error, formatstring couldn't be found, no big deal */
+
+  running=strchr(running,'\n'); // find newline
+  if (running==NULL)
+    return -1; /* again non fatal error */
+
+  running=running+1;
+  if (running=='\0') /* if we reached the end of the text, means no data available, but this is not always the case, see below */
+    return -1; /* again non fatal error */
+
+  /* 
+   * sometimes there's more text but no actual valid data
+   * one can't blindly rely upon data provided by this (or any) webservice to be in a specific fixed format
+   * in case formatstring is larger than whatever text is left
+   * then it's a sure bet we should not do further processing and risk a crash 
+   * the old data is thus prerserved and re-used
+   */
+  if ((strlen(running)) < (strlen(equakedata->formatstring)))
+    return -1;
+
+  /* 
+   * we will load the data into the array up to a maxmimum of MAXQUAKES_HR
+   * although it's unlikely that maximum would ever be reached
+   */
+  for(n=0;n<MAXQUAKES_WEEK;n++)
+  {
+    token=strsep(&running, delimiters);
+    if (token==NULL)
+      break;
+
+    /* lets make date more readable by filtering out T and Z strings and adding some clarification */
+    dt1=strsep(&token, delimiterdate);
+    if (dt1==NULL)
+      break;
+    dt2=strsep(&token, delimiterdate);
+    if (dt2==NULL)
+      break;
+    snprintf(buf, URLSIZE, "%s%s%s%s%s", dt1, "  ", dt2, " ", TIMEZONE);
+
+    equakedata->equakeformatweek[n].datetime=strndup(buf, strlen(buf));
+
+    if (equakedata->equakeformatweek[n].datetime!=NULL)
+    {
+      /* now that we have datetime lets convert it to local time */
+      if (convert_localtime(&equakedata->equakeformatweek[n])==-1)
+        equakedata->equakeformatweek[n].datetime_local=strndup(buf, strlen(buf));
+
+      token=strsep(&running, delimiters);
+      if (token==NULL)
+        break;
+
+      equakedata->equakeformatweek[n].lat=strndup(token, strlen(token));
+      if (equakedata->equakeformatweek[n].lat!=NULL)
+      {
+        token=strsep(&running, delimiters);
+        if (token==NULL)
+          break;
+
+        equakedata->equakeformatweek[n].lon=strndup(token, strlen(token));
+        if (equakedata->equakeformatweek[n].lon!=NULL)
+        {
+          token=strsep(&running, delimiters); // eliminate whitespace 
+          if (token==NULL)
+            break;
+
+          equakedata->equakeformatweek[n].depth=strndup(token, strlen(token));
+          if (equakedata->equakeformatweek[n].depth!=NULL)
+          {
+            token=strsep(&running, delimiters); // eliminate whitespace
+            if (token==NULL)
+              break;
+
+            equakedata->equakeformatweek[n].magnitude=strndup(token, strlen(token));
+            if (equakedata->equakeformatweek[n].magnitude!=NULL)
+            {
+              token=strsep(&running, delimiters);
+              if (token==NULL)
+                break;
+
+              equakedata->equakeformatweek[n].magnitude_type=strndup(token, strlen(token));
+              if (equakedata->equakeformatweek[n].magnitude_type!=NULL)
+              {
+                token=strsep(&running, delimiters);
+                if (token==NULL)
+                  break;
+
+                equakedata->equakeformatweek[n].nst=strndup(token, strlen(token));
+                if (equakedata->equakeformatweek[n].nst!=NULL)
+                {
+                  token=strsep(&running, delimiters);
+                  if (token==NULL)
+                    break;
+
+                  equakedata->equakeformatweek[n].gap=strndup(token, strlen(token));
+                  if (equakedata->equakeformatweek[n].gap!=NULL)
+                  {
+                    token=strsep(&running, delimiters); // eliminate whitespace
+                    if (token==NULL)
+                      break;
+
+                    equakedata->equakeformatweek[n].dmin=strndup(token, strlen(token));
+                    if (equakedata->equakeformatweek[n].dmin!=NULL)
+                    {
+                      token=strsep(&running, delimiters); // eliminate whitespace
+                      if (token==NULL)
+                        break;
+
+                      equakedata->equakeformatweek[n].rms=strndup(token, strlen(token));
+                      if (equakedata->equakeformatweek[n].rms!=NULL)
+                      {
+                        token=strsep(&running, delimiters); // eliminate whitespace
+                        if (token==NULL)
+                          break;
+
+                        equakedata->equakeformatweek[n].src=strndup(token, strlen(token));
+                        if (equakedata->equakeformatweek[n].src!=NULL)
+                        {
+                          token=strsep(&running, delimiters); // eliminate whitespace
+                          if (token==NULL)
+                            break;
+
+                          equakedata->equakeformatweek[n].eqid=strndup(token, strlen(token));
+                          if (equakedata->equakeformatweek[n].eqid!=NULL)
+                          {
+                            token=strsep(&running, delimiters); // eliminate whitespace
+                            if (token==NULL)
+                              break;
+
+                            equakedata->equakeformatweek[n].updated=strndup(token, strlen(token));
+                            if (equakedata->equakeformatweek[n].updated!=NULL)
+                            {
+                              token=strsep(&running, delimiters); // eliminate whitespace
+                              if (token==NULL)
+                                break;
+
+                              token=strsep(&running, delimiterquote); // this will save what's in between quotes
+                              if (token==NULL)
+                                break;
+
+                              equakedata->equakeformatweek[n].region=strndup(token, strlen(token));
+                              if (equakedata->equakeformatweek[n].region=="")
+                                return -1;
+
+                              /* 
+                               * we find a newline and then try to go one step after that which is the start of the
+                               * next line, or if that fails we reached the end of the buffer
+                               */
+                              running=strchr(running,'\n'); // find newline
+                              if (running==NULL)
+                                break; /* again non fatal error */
+
+                              running=running+1;
+                              if (running=='\0') /* if we reached the end of the text, means no data available, but this is not always the case, see below */
+                                break;
+                            }
+                          }
+                        }
+                      }
+                    }
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+  equakedata->quake_history_countweekly=n;
+
+  return equakedata->quake_history_countweekly;
+}
   
 int render_equake_label(struct Equake_Data *equakedata)
 {
@@ -283,8 +488,9 @@ int render_equake_label(struct Equake_Data *equakedata)
 		else
 		  snprintf(equakedata->lbl, LABELSIZE, "%s %s", equakedata->equakeformatday[i].magnitude, equakedata->equakeformatday[i].region);
 
-		snprintf(equakedata->tooltiptext_label, URLSIZE, "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km", equakedata->equakeformatday[i].magnitude, equakedata->equakeformatday[i].datetime, equakedata->equakeformatday[i].region, equakedata->equakeformatday[i].lat, equakedata->equakeformatday[i].lon, equakedata->equakeformatday[i].depth);
+		snprintf(equakedata->tooltiptext_label, URLSIZE, "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km", equakedata->equakeformatday[i].magnitude, equakedata->equakeformatday[i].datetime, equakedata->equakeformatday[i].datetime_local, equakedata->equakeformatday[i].region, equakedata->equakeformatday[i].lat, equakedata->equakeformatday[i].lon, equakedata->equakeformatday[i].depth);
 		equakedata->posperiod=DAILY;
+
 		return i;
               }
 	    }
@@ -296,8 +502,6 @@ int render_equake_label(struct Equake_Data *equakedata)
           return -1;
       }
     }
-    else
-      i=0;
 
     if (equakedata->equakeformat[i].src!="")
       snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformat[i].eqid, QUAKEPATH_AF);
@@ -308,8 +512,9 @@ int render_equake_label(struct Equake_Data *equakedata)
     else
       snprintf(equakedata->lbl, LABELSIZE, "%s %s", equakedata->equakeformat[i].magnitude, equakedata->equakeformat[i].region);
 
-    snprintf(equakedata->tooltiptext_label, URLSIZE, "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km", equakedata->equakeformat[i].magnitude, equakedata->equakeformat[i].datetime, equakedata->equakeformat[i].region, equakedata->equakeformat[i].lat, equakedata->equakeformat[i].lon, equakedata->equakeformat[i].depth);
+    snprintf(equakedata->tooltiptext_label, URLSIZE, "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km", equakedata->equakeformat[i].magnitude, equakedata->equakeformat[i].datetime, equakedata->equakeformat[i].datetime_local, equakedata->equakeformat[i].region, equakedata->equakeformat[i].lat, equakedata->equakeformat[i].lon, equakedata->equakeformat[i].depth);
     equakedata->posperiod=HOURLY;
+
     return i;
 
   }
@@ -354,8 +559,9 @@ void set_signature_file(struct Equake_Data *equakedata)
 int processdata(struct Equake_Data *equakedata, int period)
 {
   const char delimiters[] = ",\"";
-  const char delimiterquote[] = "\""; // some fields are between quotes, with commas
-  char *token;
+  const char delimiterquote[] = "\""; /* some fields are between quotes, with commas */
+  const char delimiterdate[]="TZ"; /* new date format has strings T and Z, we use it to separate date and time */
+  char *token, *dt1, *dt2, buf[URLSIZE];
   int n, pos=0;
   double mag;
  
@@ -368,6 +574,11 @@ int processdata(struct Equake_Data *equakedata, int period)
     n=processdata_daily(equakedata);
     return n;
   }
+  if (period==WEEKLY)
+  {
+    n=processdata_weekly(equakedata);
+    return n;
+  }
  
   equakedata->quake_history_count=-1; 
   /*
@@ -376,7 +587,7 @@ int processdata(struct Equake_Data *equakedata, int period)
    */
   char *running=alloca(sizeof(equakedata->buffer)+1);
   if (running==NULL)
-    exit(-1);
+    return -1;
 
   snprintf(running, strlen(equakedata->buffer), "%s", equakedata->buffer);
 
@@ -419,92 +630,106 @@ int processdata(struct Equake_Data *equakedata, int period)
     if (token==NULL)
       break;
 
-    equakedata->equakeformat[n].datetime=strndup(token, strlen(token));
-    if (equakedata->equakeformat[n].datetime!="")
+    /* lets make date more readable by filtering out T and Z strings and adding some clarification */
+    dt1=strsep(&token, delimiterdate);
+    if (dt1==NULL)
+      break;
+    dt2=strsep(&token, delimiterdate);
+    if (dt2==NULL)
+      break;
+    snprintf(buf, URLSIZE, "%s%s%s%s%s", dt1, "  ", dt2, " ", TIMEZONE);
+
+    equakedata->equakeformat[n].datetime=strndup(buf, strlen(buf));
+
+    if (equakedata->equakeformat[n].datetime!=NULL)
     {
+      /* now that we have datetime lets convert it to local time */
+      if (convert_localtime(&equakedata->equakeformat[n])==-1)
+        equakedata->equakeformat[n].datetime_local=strndup(buf, strlen(buf));
+
       token=strsep(&running, delimiters);
       if (token==NULL)
         break;
 
       equakedata->equakeformat[n].lat=strndup(token, strlen(token));
-      if (equakedata->equakeformat[n].lat!="")
+      if (equakedata->equakeformat[n].lat!=NULL)
       {
         token=strsep(&running, delimiters);
         if (token==NULL)
           break;
 
         equakedata->equakeformat[n].lon=strndup(token, strlen(token));
-        if (equakedata->equakeformat[n].lon!="")
+        if (equakedata->equakeformat[n].lon!=NULL)
         {
           token=strsep(&running, delimiters); // eliminate whitespace 
           if (token==NULL)
             break;
 
           equakedata->equakeformat[n].depth=strndup(token, strlen(token));
-          if (equakedata->equakeformat[n].depth!="")
+          if (equakedata->equakeformat[n].depth!=NULL)
           {
             token=strsep(&running, delimiters); // eliminate whitespace
             if (token==NULL)
               break;
 
             equakedata->equakeformat[n].magnitude=strndup(token, strlen(token));
-            if (equakedata->equakeformat[n].magnitude!="")
+            if (equakedata->equakeformat[n].magnitude!=NULL)
             {
               token=strsep(&running, delimiters);
               if (token==NULL)
                 break;
 
               equakedata->equakeformat[n].magnitude_type=strndup(token, strlen(token));
-              if (equakedata->equakeformat[n].magnitude_type!="")
+              if (equakedata->equakeformat[n].magnitude_type!=NULL)
               {
                 token=strsep(&running, delimiters);
                 if (token==NULL)
                   break;
 
                 equakedata->equakeformat[n].nst=strndup(token, strlen(token));
-                if (equakedata->equakeformat[n].nst!="")
+                if (equakedata->equakeformat[n].nst!=NULL)
                 {
                   token=strsep(&running, delimiters);
                   if (token==NULL)
                     break;
 
                   equakedata->equakeformat[n].gap=strndup(token, strlen(token));
-                  if (equakedata->equakeformat[n].gap!="")
+                  if (equakedata->equakeformat[n].gap!=NULL)
                   {
                     token=strsep(&running, delimiters); // eliminate whitespace
                     if (token==NULL)
                       break;
 
                     equakedata->equakeformat[n].dmin=strndup(token, strlen(token));
-                    if (equakedata->equakeformat[n].dmin!="")
+                    if (equakedata->equakeformat[n].dmin!=NULL)
                     {
                       token=strsep(&running, delimiters); // eliminate whitespace
                       if (token==NULL)
                         break;
 
                       equakedata->equakeformat[n].rms=strndup(token, strlen(token));
-                      if (equakedata->equakeformat[n].rms!="")
+                      if (equakedata->equakeformat[n].rms!=NULL)
                       {
                         token=strsep(&running, delimiters); // eliminate whitespace
                         if (token==NULL)
                           break;
 
                         equakedata->equakeformat[n].src=strndup(token, strlen(token));
-                        if (equakedata->equakeformat[n].src!="")
+                        if (equakedata->equakeformat[n].src!=NULL)
                         {
                           token=strsep(&running, delimiters); // eliminate whitespace
                           if (token==NULL)
                             break;
 
                           equakedata->equakeformat[n].eqid=strndup(token, strlen(token));
-                          if (equakedata->equakeformat[n].eqid!="")
+                          if (equakedata->equakeformat[n].eqid!=NULL)
                           {
                             token=strsep(&running, delimiters); // eliminate whitespace
                             if (token==NULL)
                               break;
 
                             equakedata->equakeformat[n].updated=strndup(token, strlen(token));
-                            if (equakedata->equakeformat[n].updated!="")
+                            if (equakedata->equakeformat[n].updated!=NULL)
                             {
                               token=strsep(&running, delimiters); // eliminate whitespace
                               if (token==NULL)
@@ -576,26 +801,26 @@ int processdata(struct Equake_Data *equakedata, int period)
         }
       }
     } 
-    else
+  }
+  else
+  {
+    if (equakedata->equakeformatday[pos].magnitude!=NULL)
     {
-      if (equakedata->equakeformatday[pos].magnitude!=NULL)
-      {
-        mag=atof(equakedata->equakeformatday[pos].magnitude);
-        if (equakedata->sigfilemag>0)
-        { 
-          if (mag>=equakedata->sigfilemag)
+      mag=atof(equakedata->equakeformatday[pos].magnitude);
+      if (equakedata->sigfilemag>0)
+      { 
+        if (mag>=equakedata->sigfilemag)
+        {
+          if ((equakedata->equakeformatday[pos].src!="") && (strncmp(equakedata->sigfile_prev_eqid, equakedata->equakeformatday[pos].eqid, strlen(equakedata->equakeformatday[pos].eqid))))
           {
-            if ((equakedata->equakeformatday[pos].src!="") && (strncmp(equakedata->sigfile_prev_eqid, equakedata->equakeformatday[pos].eqid, strlen(equakedata->equakeformatday[pos].eqid))))
-            {
-              set_signature_file(equakedata);
-              /* store new eqid into sigfile_prev_eqid */
-              equakedata->sigfile_prev_eqid=strndup(equakedata->equakeformatday[pos].eqid, strlen(equakedata->equakeformatday[pos].eqid));
-            }
+            set_signature_file(equakedata);
+            /* store new eqid into sigfile_prev_eqid */
+            equakedata->sigfile_prev_eqid=strndup(equakedata->equakeformatday[pos].eqid, strlen(equakedata->equakeformatday[pos].eqid));
           }
         }
-      } 
-    }
-   }
+      }
+    } 
+  }
   /* set the icon depending on the magnitude */
   set_magnitude_icon(equakedata, pos);
 
@@ -608,9 +833,10 @@ int processdata(struct Equake_Data *equakedata, int period)
 
 void alert_heavy_quake(struct Equake_Data *equakedata, int pos)
 {
-  int n;
   double mag;
   char *alert=alloca(sizeof(equakedata->buffer)+1); /* there's not more memory needed than what's in the buffer and normally much less */
+  if (alert==NULL)
+    return;
   char url[URLSIZE];
 
   if (equakedata->posperiod==HOURLY)
@@ -630,7 +856,7 @@ void alert_heavy_quake(struct Equake_Data *equakedata, int pos)
 	{
 	  snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformat[pos].eqid, QUAKEPATH_AF);
 
-	  snprintf(alert, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformat[pos].magnitude, equakedata->equakeformat[pos].datetime, equakedata->equakeformat[pos].region, equakedata->equakeformat[pos].lat, equakedata->equakeformat[pos].lon, equakedata->equakeformat[pos].depth, url);
+	  snprintf(alert, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformat[pos].magnitude, equakedata->equakeformat[pos].datetime, equakedata->equakeformat[pos].datetime_local, equakedata->equakeformat[pos].region, equakedata->equakeformat[pos].lat, equakedata->equakeformat[pos].lon, equakedata->equakeformat[pos].depth, url);
 	  equakedata->report_prev_eqid=strndup(equakedata->equakeformat[pos].eqid, strlen(equakedata->equakeformat[pos].eqid));
 	
 	  /* this is not user selected, but automatically pops up, so we don't want to block the rest of the app */ 
@@ -656,7 +882,7 @@ void alert_heavy_quake(struct Equake_Data *equakedata, int pos)
         {
           snprintf(url, URLSIZE, "%s%s%s%s%s", "http://", equakedata->quakeserver, equakedata->quakepath_ww, equakedata->equakeformatday[pos].eqid, QUAKEPATH_AF);
 
-          snprintf(alert, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformatday[pos].magnitude, equakedata->equakeformatday[pos].datetime, equakedata->equakeformatday[pos].region, equakedata->equakeformatday[pos].lat, equakedata->equakeformatday[pos].lon, equakedata->equakeformatday[pos].depth, url);
+          snprintf(alert, strlen(equakedata->buffer), "Earthquake Magnitude: %s\nDate: %s\nDate Local: %s\nLocation: %s\nLatitude: %s; Longitude: %s\nDepth: %s km\nurl: %s\n\n", equakedata->equakeformatday[pos].magnitude, equakedata->equakeformatday[pos].datetime, equakedata->equakeformatday[pos].datetime_local, equakedata->equakeformatday[pos].region, equakedata->equakeformatday[pos].lat, equakedata->equakeformatday[pos].lon, equakedata->equakeformatday[pos].depth, url);
           equakedata->report_prev_eqid=strndup(equakedata->equakeformatday[pos].eqid, strlen(equakedata->equakeformatday[pos].eqid));
 
           /* this is not user selected, but automatically pops up, so we don't want to block the rest of the app */
@@ -690,9 +916,19 @@ void set_magnitude_icon(struct Equake_Data *equakedata, int pos)
 
   /* convert magnitude to double */
   if (equakedata->posperiod==HOURLY)
-    mag=atof(equakedata->equakeformat[pos].magnitude);
+  {
+    if (equakedata->equakeformat[pos].magnitude!=NULL)
+      mag=atof(equakedata->equakeformat[pos].magnitude);
+    else
+      mag=0;
+  }
   else
-    mag=atof(equakedata->equakeformatday[pos].magnitude);
+  {
+    if (equakedata->equakeformatday[pos].magnitude!=NULL)
+      mag=atof(equakedata->equakeformatday[pos].magnitude);
+    else
+      mag=0;
+  }
 
   if (mag<2)
     quake_logo=gdk_pixbuf_new_from_inline(-1, mag1, FALSE, NULL);
@@ -719,105 +955,96 @@ void set_magnitude_icon(struct Equake_Data *equakedata, int pos)
 }
 
 /*
- * TODO fix
- * this is supposed to convert UTC time into local time
- * however it doesn't look like it does really
+ * Fixed, see: http://stackoverflow.com/questions/9076494/how-to-convert-from-utc-to-local-time-in-c
+ * this converts UTC time into local time
  */
-int convert_localtime(struct Equake_Data *equakedata)
+int convert_localtime(struct Equake_Format *equakeformat)
 {
-  char *c[7];
+  char *c[8];
   char *token;
-  const char delimitercolon[] = ", :";
-  time_t utc;
+  const char delimitercolon[] = "-:. ";
+  time_t utc, local;
 
-  struct tm *tp=malloc(sizeof(struct tm));
+  struct tm *tp=alloca(sizeof(struct tm));
   if(tp==NULL)
-    exit(-1);
+    return -1;
 
   struct tm *localt=alloca(sizeof(struct tm));
   if(localt==NULL)
-    exit(-1);
+    return -1;
 
-  char *datetime=alloca(sizeof(equakedata->equakeformat[0].datetime)+1);
+  char *datetime=alloca(URLSIZE+1); /* we use URLSIZE since it's also as the size used to allocate for equakeformat->datetime */
   if(datetime==NULL)
-    exit(-1);
+    return -1;
 
-  char *printtime=alloca(sizeof(equakedata->equakeformat[0].datetime)+1);
+  char *printtime=alloca(URLSIZE+1);
   if(printtime==NULL)
-    exit(-1);
+    return -1;
 
-  char *running2=alloca(sizeof(equakedata->equakeformat[0].datetime)+1);
+  char *running2=alloca(URLSIZE+1);
   if (running2==NULL)
-    exit(-1);
+    return -1;
 
   memset(tp, 0, sizeof(struct tm));
   memset(localt, 0, sizeof(struct tm));
+  snprintf(running2, URLSIZE, "%s", equakeformat->datetime);
 
-  sprintf(running2, "%s", equakedata->equakeformat[0].datetime);
-
-  token=strsep(&running2, delimitercolon); // day of week
-  if (token!="")
+  token=strsep(&running2, delimitercolon); /* year */
+  if (token!=NULL)
   {
     c[0]=strndup(token, strlen(token));
-    if (c[0]=="")
-      return 0;
-    token=strsep(&running2, delimitercolon); // eliminate whitespace
-    token=strsep(&running2, delimitercolon); // month
+    if (c[0]==NULL)
+      return -1;
+    token=strsep(&running2, delimitercolon); /* month */
 
-    if (token!="")
+    if (token!=NULL)
     {
       c[1]=strndup(token, strlen(token));
-      if (c[1]=="")
-        return 0;
-      token=strsep(&running2, delimitercolon); // day of month
-      
-      if (token==" ") /* if day of month is between 1 and 9 then there is one extra space */
-	token=strsep(&running2, delimitercolon); /* this is real day of month when day is between 1 and 9 */
-      else
-        token=strsep(&running2, delimitercolon); // eliminate whitespace
-      
-      if (token!="")
+      if (c[1]==NULL)
+        return -1;
+      token=strsep(&running2, delimitercolon); /* day of month */
+      if (token!=NULL)
       {
         c[2]=strndup(token, strlen(token));
-	if (c[2]=="")
-          return 0;
-        /*token=strsep(&running2, delimitercolon); // eliminate whitespace*/
-        token=strsep(&running2, delimitercolon); // year
-
-        if (token!="")
+        if (c[2]==NULL)
+          return -1;
+        token=strsep(&running2, delimitercolon); /* eliminate whitespace */
+        if (token==NULL)
+          return -1;
+        token=strsep(&running2, delimitercolon); /* hours */
+        if (token==NULL)
+          return -1;
+        if (token!=NULL)
         {
           c[3]=strndup(token, strlen(token));
-	  if (c[3]=="")
-	    return 0;
-          token=strsep(&running2, delimitercolon); /* hours */
-
-          if (token!="")
+          if (c[3]==NULL)
+            return -1;
+          token=strsep(&running2, delimitercolon); /* minutes */
+          if (token!=NULL)
           {
             c[4]=strndup(token, strlen(token));
-	    if (c[4]=="")
-	      return 0;
-            token=strsep(&running2, delimitercolon); /* minutes */
-
-            if (token!="")
+            if (c[4]==NULL)
+              return -1;
+            token=strsep(&running2, delimitercolon); /* seconds */
+            if (token!=NULL)
             {
               c[5]=strndup(token, strlen(token));
-	      if (c[5]=="")
-		return 0;
-              token=strsep(&running2, delimitercolon); /* seconds */
-
-              if (token!="")
+              if (c[5]==NULL)
+                return -1;
+              token=strsep(&running2, delimitercolon); /* mseconds */
+              if (token!=NULL)
               {
                 c[6]=strndup(token, strlen(token));
-		if (c[6]=="")
-	          return 0;
+                if (c[6]==NULL)
+                  return -1;
                 token=strsep(&running2, delimitercolon); /* timezone */
-
-                if (token!="")
+                if (token!=NULL)
                 {
                   c[7]=strndup(token, strlen(token));
-	          if (c[7]=="")
-	            return 0;
-                  sprintf(datetime, "%s %s %s %s %s %s %s %s", c[0], c[1], c[2], c[3], c[4], c[5], c[6], c[7]);
+                  if (c[7]==NULL)
+                    return -1;
+                  /* we will not record mseconds here since strptime() doesn't have it */
+                  snprintf(datetime, URLSIZE, "%s %s %s %s %s %s %s", c[0], c[1], c[2], c[3], c[4], c[5], c[7]);
                 }
               }
             }
@@ -826,32 +1053,30 @@ int convert_localtime(struct Equake_Data *equakedata)
       }
     }
   }
+  strptime(datetime, "%Y %m %d %H %M %S %z", tp); /* year month day hour minute second */
 
-  strptime(datetime, "%A %B %d %Y %H %M %S %Z", tp);
-  /*strftime(printtime, strlen(datetime), "%A %B %d %Y %H %M %S %Z", tp);*/
-  printf("%s\n", datetime);
   /* convert UTC in seconds since EPOCH */
   utc=mktime(tp);
-  /*printtime=ctime(&utc);*/
+
+  printtime=ctime(&utc);
 
   char buf[100];
   struct tm e0 = { .tm_year = 70, .tm_mday = 1 }, e1, new;
+
   /* get time_t EPOCH value for e0 (Jan. 1, 1970) */
   time_t pseudo = mktime(&e0);
+
   /* get gmtime for this value */
   e1 = *gmtime(&pseudo);
-
-  /* get local time in seconds since EPOCH using values found above */
   e0.tm_sec += utc - diff_tm(&e1, &e0);
-  mktime(&e0);
+  local=e0.tm_sec;
+  localt=localtime(&local);
+
   /* get human readable time */
-  strftime(buf, sizeof buf, "%c", &e0); 
-  /*puts(buf);*/
-  printf("%s\n", buf);
+  strftime(buf, sizeof buf, "%Y-%m-%d %H:%M:%S %Z", localt);
+  equakeformat->datetime_local=strndup(buf, strlen(buf));
 
-  equakedata->equakeformat[0].datetime_local=strndup(buf, strlen(buf));
-
-  return 1;
+  return 0;
 }
 
 long long diff_tm(struct tm *a, struct tm *b)
